/*
 *  yuv2ppm.c
 *
 * Convert YUV422 captured by NetWinder vidcap to RGB24 PPM.
 *
 *  Copyright (C) 1995-2000 Russell King.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <unistd.h>

#include "yuvtorgb.h"

static unsigned char *map_fd(int fd, int *size)
{
	caddr_t addr;
	struct stat buf;

	if (fstat(fd, &buf))
		return NULL;

	*size = buf.st_size;

	addr = mmap(0, buf.st_size, PROT_READ, MAP_FILE|MAP_SHARED, fd, 0);
	if (addr == (caddr_t)-1)
		return NULL;

	return (unsigned char *)addr;
}

int main(int argc, char *argv[])
{
	unsigned char *inbuf, *outbuf;
	int fdin, fdout, size;

	if (argc != 3) {
		fprintf(stderr, "Syntax: out in\n");
		exit(1);
	}

	yuv422_to_rgb_init();

	fdin = open(argv[1], O_RDONLY);
	if (fdin < 0) {
		perror("open");
		exit(1);
	}

	inbuf = map_fd(fdin, &size);
	if (!inbuf) {
		perror("mmap");
		exit(1);
	}
	close(fdin);

	outbuf = malloc(size * 6 / 4);
	if (!outbuf) {
		perror("malloc");
		exit(1);
	}

	yuv422_to_rgb24(outbuf, inbuf, size);

	if (strcmp(argv[2], "-") == 0)
		fdout = 1;
	else {
		fdout = open(argv[2], O_CREAT|O_TRUNC|O_WRONLY, 0666);
		if (fdout < 0) {
			perror("open");
			exit(1);
		}
	}

	write(fdout, "P6\n716 576\n255\n", 15);
	write(fdout, outbuf, size * 6 / 4);

	if (fdout != 1)
		close(fdout);
	exit(0);
}
